#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# opensecurity_service
# 
# turn the opensecurityd into a Windows Service
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright 2013-2014 X-Net and AIT Austrian Institute of Technology
# 
# 
#     X-Net Services GmbH
#     Elisabethstrasse 1
#     4020 Linz
#     AUSTRIA
#     https://www.x-net.at
# 
#     AIT Austrian Institute of Technology
#     Donau City Strasse 1
#     1220 Wien
#     AUSTRIA
#     http://www.ait.ac.at
# 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#    http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import os
import os.path
import time
import servicemanager
import urllib2
import win32event
import win32service
import win32serviceutil

from opensecurity_util import logger
import opensecurityd


# ------------------------------------------------------------
# code


class OpenSecurityService(win32serviceutil.ServiceFramework):
    
    """This is the OpenSecurity Windows Service,"""

    _svc_name_ = 'OpenSecurityService'
    _svc_display_name_ = 'OpenSecurity Service'
    _svc_description_ = 'OpenSecurity Daemon Service'

    _run = True
    
    def __init__(self, args):

        """Init the Service"""
        
        # super call 
        win32serviceutil.ServiceFramework.__init__(self, args)
        self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)


    def SvcStop(self):
        
        """Stopping the Service"""

        self.log('OpenSecurity Service stopping...\n') 
        self._run = False
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)

        # as we are not running inside the
        # original service process space
        # we got to have some means to shutdown
        # the REST server from a distance ...
        #
        # TODO: find a better secure way to do that
        try:
            r = urllib2.urlopen('http://localhost:8080/terminate', timeout=1)
        except:
            pass

        win32event.SetEvent(self.hWaitStop)
        self.log('OpenSecurity Service stopped...\n') 


    def SvcDoRun(self):
    
        """Run the Service"""

        self.log('OpenSecurity Service started now')
        servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,
                              servicemanager.PYS_SERVICE_STARTED,
                              (self._svc_name_, ''))
        self._run = True
        self.main()


    def log(self, s):

        """log some string to the log file"""

        try:
            logger.debug(s)
        except:
            pass


    def main(self):

        """Main service method"""
        opensecurityd.main()


if __name__ == '__main__':
    """startup"""
    win32serviceutil.HandleCommandLine(OpenSecurityService)


